import {
  JIBESTREAM_DESTINATION,
  INSTRUCTION,
  JIBESTREAM_WAYPOINT,
  JIBESTREAM_PATH,
  JIBESTREAM_STYLE,
} from './JibestreamTypes';
import { ODP_ON_FACILITY_UPDATE } from './schemas/OdpOnFacilityUpdateType';
import { ODP_ON_KEY_VALIDATION } from './schemas/OdpOnKeyValidationType';
import { ODP_ON_NAVIGATION_UPDATE } from './schemas/OdpOnNavigationUpdateType';
import { ODP_ON_READY } from './schemas/OdpOnReadyType';

export type COMMANDS_TO_MAP =
  | 'UPDATE_USER_LOCATION'
  | 'WAYFIND_TO_WAYPOINT'
  | 'WAYFIND_BETWEEN_WAYPOINTS'
  | 'DRAW_NAVIGATION_PATH'
  | 'STYLE_SHAPE'
  | 'RESET_ALL_UNIT_STYLES';
export type COMMANDS_FROM_MAP =
  | 'DESTINATION_SELECTED'
  | 'NAVIGATION_PATH_GENERATED'
  | 'USER_POSITION_UPDATED'
  | 'CONSOLE_LOG';
export type EVENTS_FROM_ODP = 'ON_NAVIGATION_UPDATE' | 'ON_FACILITY_UPDATE';

export type MAP_EVENT_LISTENER = {
  onDestinationSelected?: (destination: JIBESTREAM_DESTINATION) => void;
  onNavigationPathGenerated?: (
    pathId: string,
    instructions: INSTRUCTION[],
    start: JIBESTREAM_WAYPOINT,
    end: JIBESTREAM_WAYPOINT,
    path: JIBESTREAM_PATH[]
  ) => void;
  onUserPositionUpdated?: (userPosition: number[]) => void;
};

export type ODP_EVENT_LISTENER = {
  onReady?: (message: ODP_ON_READY) => void;
  onKeyValidation?: (message: ODP_ON_KEY_VALIDATION) => void;
  onNavigationUpdate?: (message: ODP_ON_NAVIGATION_UPDATE) => void;
  onFacilityUpdate?: (message: ODP_ON_FACILITY_UPDATE) => void;
};

export type MAP_VIEW_REFERENCE = {
  setUserPosition: (lat: number, long: number) => void;
  wayfindToWaypoint: (waypointId: number) => void;
  wayfindBetweenWaypoints: (originId: number, destinationId: number) => void;
  drawNavigationPath: (
    pathId: string,
    shouldFocus?: boolean,
    style?: JIBESTREAM_STYLE
  ) => void;
  styleUnitsForDestination: (
    destination: JIBESTREAM_DESTINATION,
    style?: JIBESTREAM_STYLE
  ) => void;
  styleShape: (id: number, style?: JIBESTREAM_STYLE) => void;
};
